$MOD51

;ridici program pro barometr a teplomer na RS232
;Karel Kozlik OK1CPI - kozlik@kufr.cz
;verze 1.02

bufmax          equ             0fh
templen         equ             06h
presslen        equ             04h

buffer          equ             22h                     ; serial line input buffer
bufcnt          equ             buffer + bufmax         ; length of input string
sendptr         equ             bufcnt + 1              ; pointer to string in eprom
tick            equ             sendptr + 2
m_temp          equ             tick + 1                ; pointer to ram where is akt. temperature
m_press         equ             m_temp + templen        ; pointer to ram where is akt. pressure

var1            equ             m_press + presslen
var2            equ             var1 + 4
var3            equ             var2 + 4
var4            equ             var3 + 4
var5            equ             var4 + 4
var6            equ             var5 + 4
stack           equ             var6 + 4

t_aver          equ             10h
t_avcnt         equ             t_aver + 5
p_aver          equ             t_avcnt + 1
p_avcnt         equ             p_aver + 5


f_bufvalid      bit             1       ; line was received or should be sent
f_sendcode      bit             2       ; send constant string from pointer sendptr
f_rcvr_busy     bit             3       ; receiver is busy while sending data
f_sndeol        bit             4       ; when set, we are sending "\r\n"
f_sndbsp        bit             5       ; when set, we echo "\b \b" for clearing one character
f_first         bit             6       ; counter of second and third character of backspace, second of eof
f_temp_conected bit             7       ; is conected temp sensor?
f_mesure_break  bit             8       ; temp mesure was breaked
f_disdata       bit             9       ; should be send data to display

p_disp7         bit             p1.7
p_disp6         bit             p1.6
p_disp5         bit             p1.5
p_disp4         bit             p1.4
p_dispE         bit             p1.3
p_dispRS        bit             p1.2
p_ADclk         bit             p1.1
p_ADout         bit             p1.0

p_ADcs          bit             p3.7
p_nonexisting   bit             p3.6    ; does not exist on AT89C2051 (output from internal comparator)
p_enh1          bit             p3.5
p_enh2          bit             p3.4
p_enh3          bit             p3.3
p_temp          bit             p3.2

                org             0000h
                jmp             start
                org             000bh
                jmp             ticker
                org             0023h
                jmp             serial

$include(MATH1.ASM)

start:          mov             sp,#stack
                mov             tick,#13

                mov             t_avcnt,#16

                mov             t_aver,#0
                mov             t_aver+1,#0
                mov             t_aver+2,#0
                mov             t_aver+3,#0
                mov             t_aver+4,#0

                mov             p_avcnt,#16

                mov             p_aver,#0
                mov             p_aver+1,#0
                mov             p_aver+2,#0
                mov             p_aver+3,#0
                mov             p_aver+4,#0

                mov             m_temp,#'-'
                mov             m_temp+1,#'-'
                mov             m_temp+2,#'-'
                mov             m_temp+3,#'-'
                mov             m_temp+4,#'-'
                mov             m_temp+5,#'-'

                mov             m_press,#'-'
                mov             m_press+1,#'-'
                mov             m_press+2,#'-'
                mov             m_press+3,#'-'

                clr             rs0
                clr             rs1

                mov             scon, #50h      ; 8bit UART
                mov             tmod, #21h      ; timer 0 - mod 1 ,timer 1 - mod 2
                mov             th1, #0fbh      ; 9600bd for quartz 18.432MHz
                anl             pcon, #7Fh
                mov             ie, #92h        ; Interrupt enable from timer0 and serial controller
                mov             ip, #10h        ; Serial line has high priority
                setb            tr0             ; run ticker
;                clr            tr0             ; stop ticker
                setb            tr1             ; run timer1

                clr             f_bufvalid
                clr             f_sendcode
                clr             f_rcvr_busy
                clr             f_sndeol
                clr             f_sndbsp
                clr             f_first
                clr             f_temp_conected
                clr             f_mesure_break

                mov             bufcnt, #0

                setb            f_rcvr_busy     ; send version message
                mov             dptr, #version_text
                mov             sendptr, dph
                mov             sendptr+1, dpl
                setb            f_sendcode
                acall            ser_send

cyc:            setb            tr0
                jnb             f_bufvalid, $
                clr             tr0
                acall           del_white
                acall           parse_cmds
                jb              f_rcvr_busy, $  ; wait while not send
                jmp             cyc

;; **************************** del_white **********************************
del_white:      push            acc
                mov             a,r0
                push            acc
                mov             a,r1
                push            acc
                mov             a,r2
                push            acc
                mov             a,r3
                push            acc

                mov             r0, #buffer
                mov             r1, #buffer
                mov             r2, #0          ; counter of copied chars
                mov             r3, bufcnt
                cjne            r3, #0, del_whloop
                jmp             del_whend
del_whloop:     mov             a, @r0
                inc             r0
                cjne            a, #' ', del_whl1
                jmp             del_whll
del_whl1:       cjne            a, #9, del_whl2
                jmp             del_whll
del_whl2:       cjne            a, #10, del_whl3
                jmp             del_whll
del_whl3:       cjne            a, #13, del_whl4
                jmp             del_whll
del_whl4:       mov             @r1, a
                inc             r1
                inc             r2
del_whll:       djnz            r3, del_whloop
del_whend:      mov             bufcnt, r2

                pop             acc
                mov             r3,a
                pop             acc
                mov             r2,a
                pop             acc
                mov             r1,a
                pop             acc
                mov             r0,a
                pop             acc
                ret

;; **************************** parse_cmds *********************************
parse_cmds:     push            acc
                mov             a,r0
                push            acc
                mov             a,r1
                push            acc
                push            dpl
                push            dph

                mov             r1, bufcnt
                cjne            r1, #0, parse_nozero
                clr             f_bufvalid
                clr             f_rcvr_busy
                jmp             parse_ed2

parse_nozero:   mov             r0, #buffer
                mov             a, @r0
                mov             r0, #buffer+1
                mov             r1, bufcnt
                dec             r1

parse_help:     cjne            a, #'H', parse_temp
                cjne            r1, #0, parse_error     ; 'H'
                clr             f_bufvalid
                mov             dptr, #help_text
                jmp             parse_end

parse_temp:     cjne            a, #'T', parse_pressure
                cjne            r1, #0, parse_error     ; 'T'
                clr             f_bufvalid
                acall            pars_temp2
                jmp             parse_ed2

parse_pressure: cjne            a, #'P', parse_ver
                cjne            r1, #0, parse_error     ; 'P'
                clr             f_bufvalid
                acall            pars_press2
                jmp             parse_ed2

parse_ver:      cjne            a, #'V', parse_error     ; 'V'
                cjne            r1, #0, parse_error
                clr             f_bufvalid
                mov             dptr, #version_text
                jmp             parse_end

parse_error:    clr             f_bufvalid
                mov             dptr, #err_text
                jmp             parse_end

parse_end:      mov             sendptr, dph
                mov             sendptr+1, dpl
                setb            f_sendcode
                acall            ser_send

parse_ed2:      mov             bufcnt, #0
                pop             dph
                pop             dpl
                pop             acc
                mov             r1,a
                pop             acc
                mov             r0,a
                pop             acc
                ret

;; ********************************** parse_press2 ********************************

pars_press2:    push            acc
                push            dpl
                push            dph

                mov             buffer, m_press
                mov             buffer+1, m_press+1
                mov             buffer+2, m_press+2
                mov             buffer+3, m_press+3
                mov             bufcnt, #4
                setb            f_rcvr_busy
                setb            f_bufvalid
                acall            ser_send
                jb              f_rcvr_busy, $
                mov             dptr, #hpa_text
                mov             sendptr, dph
                mov             sendptr+1, dpl
                setb            f_sendcode
                setb            f_rcvr_busy
                acall            ser_send
                jb              f_rcvr_busy, $

                pop             dph
                pop             dpl
                pop             acc
                ret

;; ********************************** parse_temp2 ********************************

pars_temp2:     push            acc
                push            dpl
                push            dph

                mov             buffer, m_temp
                mov             buffer+1, m_temp+1
                mov             buffer+2, m_temp+2
                mov             buffer+3, m_temp+3
                mov             buffer+4, m_temp+4
                mov             buffer+5, m_temp+5
                mov             bufcnt, #6
                setb            f_rcvr_busy
                setb            f_bufvalid
                acall            ser_send
                jb              f_rcvr_busy, $
                mov             dptr, #temp_text
                mov             sendptr, dph
                mov             sendptr+1, dpl
                setb            f_sendcode
                setb            f_rcvr_busy
                acall            ser_send
                jb              f_rcvr_busy, $

                pop             dph
                pop             dpl
                pop             acc
                ret

;; ********************************** serial ********************************
;; Serial controller interrupt handler
serial:         setb            f_mesure_break
                jb              ri, ser_rcv
                acall            ser_send
                reti

;; ******************************** ser_rcv *********************************
ser_rcv:        push            psw
                push            acc
                mov             a, r0
                push            acc
                mov             a, r1
                push            acc
                mov             a, sbuf
                jb              f_rcvr_busy, rcv_end ; ignore while sending response
                cjne            a, #08d, rcv_nobsp   ; BACKSPACE?
                jmp             rcv_bsp
rcv_nobsp:      cjne            a, #7fh, rcv_nob1    ; DELETE?
                jmp             rcv_bsp
rcv_nob1:       cjne            a, #13d, rcv_noe1    ; EOL?
                jmp             rcv_eol
rcv_noe1:       cjne            a, #10d, rcv_noeol
                jmp             rcv_eol
rcv_noeol:      mov             r0, bufcnt
                cjne            r0, #bufmax, rcv_nooflow ; ignore this character?
                jmp             rcv_end
rcv_nooflow:    mov             sbuf, a         ; Echo
                clr             c               ; Upcase
                mov             r0, a
                subb            a, #'a'
                jc              upcase_end      ; a < 'a'
                clr             c
                mov             a, r0
                subb            a, #'z'+1
                jnc             upcase_end      ; a >= 'z' + 1
                clr             c
                mov             a, r0
                subb            a, #32d
                mov             r0, a
upcase_end:     mov             a, #buffer      ; put new character in buffer
                add             a, bufcnt
                mov             r1, a
                mov             a, r0
                mov             @r1, a
                inc             bufcnt
                jmp             rcv_end
rcv_bsp:        mov             a, bufcnt
                cjne            a, #0, rcv_bsp1 ; is buffer empty?
                jmp             rcv_end
rcv_bsp1:       setb            f_sndbsp
                clr             f_first
                dec             bufcnt
                mov             sbuf, #08h
                jmp             rcv_end
rcv_eol:        setb            f_sndeol
                setb            f_rcvr_busy
                clr             f_first
                mov             sbuf, #13d
rcv_end:        pop             acc
                mov             r1, a
                pop             acc
                mov             r0, a
                pop             acc
                pop             psw
                clr             ri
                reti

;; ******************************** ser_send ********************************
;; if f_sendcode -> send null ended string from eprom at sendptr
;;                  increments sendptr
;; if f_bufvalid -> send string from ram at buffer with len bufcnt
;;                  increments sendptr
;; if there is nothing to send -> enable receiver
ser_send:       push            psw
                push            acc
                mov             a, r0
                push            acc
                push            dpl
                push            dph
                jnb             f_sndeol, send_noeol    ; end of line echoed
                jb              f_first, send_eol
                setb            f_first
                mov             sbuf, #10d
                jmp             send_end
send_eol:       setb            f_bufvalid
                clr             f_sndeol
                clr             f_first
                jmp             send_end
send_noeol:     jnb             f_sndbsp, send_nobsp    ; backspace echoed
                jb              f_first, send_bsp
                setb            f_first
                mov             sbuf, #' '              ; space
                jmp             send_end
send_bsp:       clr             f_sndbsp
                clr             f_first
                mov             sbuf, #08h              ; backspace
                jmp             send_end
send_nobsp:     jnb             f_rcvr_busy, send_end
                jnb             f_sendcode, send_buffer ; should send code string?
                mov             dph, sendptr
                mov             dpl, sendptr+1
                clr             a
                movc            a, @a+dptr
                cjne            a, #0, send_code
                clr             f_sendcode
                mov             sendptr, #0
                jmp             send_buffer
send_code:      mov             sbuf, a
                inc             dptr
                mov             sendptr, dph
                mov             sendptr+1, dpl
                jmp             send_end
send_buffer:    jnb             f_bufvalid, send_finished
                mov             a, bufcnt
                cjne            a, sendptr, send_bf1
                mov             sendptr, #0
                clr             f_bufvalid
                jmp             send_finished
send_bf1:       mov             a, #buffer
                add             a, sendptr
                mov             r0, a
                mov             sbuf, @r0
                inc             sendptr
                jmp             send_end
send_finished:  mov             bufcnt, #0
                clr             f_rcvr_busy
send_end:       pop             dph
                pop             dpl
                pop             acc
                mov             r0, a
                pop             acc
                pop             psw
                clr             ti
                ret

;; ******************************* wait *********************************

wait:           mov             r5,#0
w1:             djnz            r5,w1
                ret

;; *************************** get presure ******************************
;; mesure presure and save it to m_press

get_presure:
                ;push            acc

                setb            p_ADcs          ; CS high and low
                acall            wait            ; at least 17us
                clr             p_ADcs


                mov             r1,#8           ; cycle 8x
                nop

gp_cykl8:       mov             c,p_ADout       ; this must last
                rlc             a               ; at least 1,4us

                nop
                setb            p_ADclk
                nop                             ; 400ns wait
                clr             p_ADclk

                djnz            r1,gp_cykl8     ; end of cycle
                                                ; in a is out of DA conv.

                jnz             gp_calc
                ajmp            gp_notconect

gp_calc:        mov             r1,#var1        ;clear var1 - var6
                mov             r0,#24

gp_varclr:      mov             @r1,#0
                inc             r1
                djnz            r0,gp_varclr

                                                ;100*P=80273+x*94
                                                ;      13991h  5Dh
                                                ; x .. out of AD converter
                mov             b,#05dh
                mul             ab

                mov             var1,a
                mov             var1+1,b

                mov             var2,#091h        ;var2 = 013991h = 80273
                mov             var2+1,#039h
                mov             var2+2,#01h

                mov             r0,#var1
                mov             r1,#var2
                acall           plus             ;presure is in var1

                mov             r0,#p_aver
                mov             r1,#var1
                acall           plus

                dec             p_avcnt
                mov             a,p_avcnt
                jz              gp_bcd
                ajmp            gp_end

gp_bcd:         mov             r0, #p_aver+3
                acall            div2
                mov             r0, #p_aver+3
                acall            div2
                mov             r0, #p_aver+3
                acall            div2
                mov             r0, #p_aver+3
                acall            div2

                BINBCD2          p_aver, var4

                mov             p_avcnt,#16

                mov             p_aver,#0
                mov             p_aver+1,#0
                mov             p_aver+2,#0
                mov             p_aver+3,#0
                mov             p_aver+4,#0

                mov             a,var4+2
                anl             a,#0f0h         ;thousands
                swap            a
                add             a,#48           ;to ascii
                mov             m_press,a

                mov             a,var4+2        ;hundreds
                anl             a,#0fh
                add             a,#48
                mov             m_press+1,a

                mov             a,var4+1        ;dekades
                anl             a,#0f0h
                swap            a
                add             a,#48           ; to ascii
                mov             m_press+2,a

                mov             a,var4+1        ;ones
                anl             a,#0fh
                add             a,#48
                mov             m_press+3,a

                sjmp            gp_end

gp_notconect:   mov             r1,#m_press
                mov             r0,#presslen
gp_ncloop:      mov             @r1,#45
                inc             r1
                djnz            r0,gp_ncloop

gp_end:        ; pop             acc
                ret

;; ******************************* get temp *********************************
;; measure temperature and save it in m_temp

get_temp:
                ; push            acc

                anl             ie,#11111101B   ; stop interupt from timer 0


                ; test conections

gt_avloop:      setb            f_temp_conected
                mov             r0,#4                        ;wait 2ms
gt_tst1loop:    mov             r1,#0
gt_tst2loop:    jnb             int0,gt_test2
                djnz            r1,gt_tst2loop
                djnz            r0,gt_tst1loop
                clr             f_temp_conected
                sjmp            gt_testend

gt_test2:       mov             r0,#4                        ;wait 2ms
gt_tst3loop:    mov             r1,#0
gt_tst4loop:    jb              int0,gt_testend
                djnz            r1,gt_tst4loop
                djnz            r0,gt_tst3loop
                clr             f_temp_conected

gt_testend:     jb              f_temp_conected,gt_mesure
                ajmp            gt_notconect

gt_mesure:      mov             r1,#var1                     ; clear var1 - var6
                mov             r0,#24

gt_varclr:      mov             @r1,#0
                inc             r1
                djnz            r0,gt_varclr

                mov             r0,#0
                mov             r1,#0
                mov             r3,#0
                mov             r4,#0

                mov             r2,#32          ; 32 mesures
                clr             f_mesure_break

gt_loop:        mov             tl0,r0
                mov             th0,r1
                setb            tr0
                jb              int0, $
                clr             tr0             ; signal high
                mov             r0,tl0
                mov             r1,th0

                mov             tl0,r3
                mov             th0,r4
                setb            tr0
                jnb             int0, $
                clr             tr0             ; signal low
                mov             r3,tl0
                mov             r4,th0
                djnz            r2, gt_loop

                jb              f_mesure_break,gt_mesure ;if measure was breaked will by again


                ; ulozeni high stavu do var1
                mov             a,r0
                add             a,#0e0h         ; 32 * 7 instructions - 5 =0dbh
                mov             var1,a          ;     32*7=e0h
                mov             a,r1
                addc            a,#0h
                mov             var1+1,a

                ; ulozeni low stavu do var2
                mov             a,r3
                add             a,#000h           ; 32*5 instructions = 0A0h
                mov             var2,a
                mov             a,th0
                addc            a,#0h
                mov             var2+1,a

                ; constants to var3
                mov             var3, #0a6h
                mov             var3+1,#05h     ;1446 = 05a6h

                SOUCIN          var1, var3, var5, var4       ;result in var5

                ; constants to var3
                mov             var3, #0a9h
                mov             var3+1, #02h    ;681 = 02a9h

                SOUCIN          var2, var3, var6, var4       ;result in var6

                mov             r0,#var1
                mov             r1,#var2
                acall           plus

                ; find out mark

                mov             r0, #var5
                mov             r1, #var6

                acall           cmp

                jc              gt_kladne

                mov             r0,#var6
                mov             r1,#var5
                acall           minus                          ; negative numerator to var6

                mov             a, var6
                mov             var5, a
                mov             a, var6+1
                mov             var5+1, a
                mov             a, var6+2
                mov             var5+2, a
                mov             a, var6+3
                mov             var5+3, a

                mov             m_temp, #45
                jmp             gt_hps

gt_kladne:      mov             r0,#var5
                mov             r1,#var6
                acall           minus                        ; positive numerator to var5

                mov             m_temp, #32

gt_hps:         mov             r1,#var2                     ; clear var2, var3, var4
                mov             r0,#12

gt_clrloop:     mov             @r1,#0
                inc             r1
                djnz            r0,gt_clrloop

                PODIL           var5, var1, var2, var3

                ; temperature is in var2

                mov             r0,#t_aver
                mov             r1,#var2
                acall           plus

                dec             t_avcnt
                mov             a,t_avcnt
                jz              gt_bcd
                ajmp            gt_end

gt_bcd:         mov             r0, #t_aver+3
                acall            div2
                mov             r0, #t_aver+3
                acall            div2
                mov             r0, #t_aver+3
                acall            div2
                mov             r0, #t_aver+3
                acall            div2

                BINBCD          t_aver, var4

                mov             t_avcnt,#16

                mov             t_aver,#0
                mov             t_aver+1,#0
                mov             t_aver+2,#0
                mov             t_aver+3,#0
                mov             t_aver+4,#0

                ;hundreds
                mov             a,var4+1
                anl             a,#0f0h
                swap            a
                add             a,#48
                mov             m_temp+1,a

                ;dekades
                mov             a,var4+1
                anl             a,#0fh
                add             a,#48
                mov             m_temp+2,a

                ;ones
                mov             a,var4
                anl             a,#0f0h
                swap            a
                add             a,#48
                mov             m_temp+3,a

                ;decimal point
                mov             m_temp+4,#46

                ;tenths
                mov             a,var4
                anl             a,#0fh
                add             a,#48
                mov             m_temp+5,a

                sjmp            gt_end

gt_notconect:   mov             r1,#m_temp
                mov             r0,#templen
gt_ncloop:      mov             @r1,#45
                inc             r1
                djnz            r0,gt_ncloop

gt_end:         orl             ie,#00000010B    ; set interupt from timer 0

;                pop             acc
                ret


;; *************************** ticker **************************************

ticker:         push            psw
                push            acc

                setb            rs0             ; change bank of registers r0-r7
                clr             rs1

                dec             tick
                mov             a,tick
                jnz             tickend

                clr             tr0             ; stop ticker

                acall           get_presure
                acall           get_temp

                acall           dis_init

                ; Display values

                mov             r0,#m_press
                mov             r6,#presslen
dp_loop1:       mov             b,@r0
                acall           dis_send
                inc             r0
                djnz            r6,dp_loop1

                mov             b,#'h'
                acall           dis_send
                mov             b,#'P'
                acall           dis_send
                mov             b,#'a'
                acall           dis_send

                clr             f_disdata
                mov             b,#0c0h
                acall           dis_send
                setb            f_disdata

                mov             r0,#m_temp
                mov             r6,#templen
dp_loop2:       mov             b,@r0
                acall           dis_send
                inc             r0
                djnz            r6,dp_loop2

                mov             b,#''
                acall           dis_send
                mov             b,#'C'
                acall           dis_send


                mov             tl0,#00h        ; set ticker
                mov             th0,#00h
                mov             tick,#13
                setb            tr0             ; run ticker

tickend:        clr             rs0
                clr             rs1

                pop             acc
                pop             psw
                reti

;; *************************** Displ_clock **************************************


dis_clock:      mov             r7,#91h
                djnz            r7,$
                setb            p_dispE
                mov             r7,#91h
                djnz            r7,$
                clr             p_dispE
                mov             r7,#91h
                djnz            r7,$
                ret


;; *************************** Displ_send **************************************


dis_send:       mov             a,b
                anl             a,#0f0h
                jnb             f_disdata,dis_nodata1
                add             a,#4
dis_nodata1:    add             a,#3
                mov             p1,a
                acall           dis_clock
                mov             a,b
                anl             a,#0fh
                swap            a
                jnb             f_disdata,dis_nodata2
                add             a,#4
dis_nodata2:    add             a,#3
                mov             p1,a
                acall           dis_clock
                ret

;; *************************** Displ_init **************************************

dis_init:       mov             p1,#33h
                acall           dis_clock
                mov             p1,#33h
                acall           dis_clock
                mov             p1,#33h
                acall           dis_clock
                mov             p1,#23h
                acall           dis_clock

                clr             f_disdata
                mov             b,#28h        ;4bit, 1 line, 5*10 dots
                acall           dis_send
                mov             b,#0Ch        ;disp on, curs off, curs blink off
                acall           dis_send
                mov             b,#6h         ;inc curs pos, no displ. shift
                acall           dis_send
                mov             b,#1          ;clear disp, curs home
                acall           dis_send
                mov             b,#83h        ;DDRAM 3
                acall           dis_send
                setb            f_disdata
                ret



;; *************************** strings, tables ******************************
version_text:   db   'Barometer & Thermometer for RS232', 0ah, 0dh
                db   'Ver. 1.02',0ah, 0dh
                db   'Copyright Karel Kozlik, OK1CPI, 2000', 0ah, 0dh
                db   'Type H for help', 0ah, 0dh
                db   0ah, 0dh, 00h
help_text:      db   'T - get temperature', 0ah, 0dh
                db   'P - get pressure', 0ah, 0dh
                db   'V - version', 0ah, 0dh
                db   'H - help', 0ah, 0dh
                db   0ah, 0dh, 00h

hpa_text:       db   ' hpa', 0ah, 0dh, 00h
;temp_text:      db   ' C', 0ah, 0dh, 00h
temp_text:      db   ' ',0f8h,'C', 0ah, 0dh, 00h
err_text:       db   'ERR', 0ah, 0dh, 00h

      end
